@@ -30,68 +30,131 @@ namespace KSysGuard
 
 static KLocalizedString unitFormat(Unit unit)
 {
-    const static KLocalizedString B = ki18nc("Bytes unit symbol", "%1 B");
-    const static KLocalizedString KiB = ki18nc("Kilobytes unit symbol", "%1 KiB");
-    const static KLocalizedString MiB = ki18nc("Megabytes unit symbol", "%1 MiB");
-    const static KLocalizedString GiB = ki18nc("Gigabytes unit symbol", "%1 GiB");
-    const static KLocalizedString TiB = ki18nc("Terabytes unit symbol", "%1 TiB");
-    const static KLocalizedString PiB = ki18nc("Petabytes unit symbol", "%1 PiB");
+    // The strings here use \u200B (zero-width space) to allow the unit symbol
+    // to be wrapped to a new line when there is little space and potentially
+    // even be hidden that way. Since the prefix is part of the unit it risks
+    // hiding the prefix, so the zero-width space is placed after the prefix and
+    // we use \u202F (narrow no-break space) to ensure we have some separation
+    // but do not wrap it. For the prefix-less units where we *do* want to allow
+    // wrapping, we use \u2009 (thin space) to ensure we have the same spacing
+    // between prefix-less and prefixed units.
 
-    const static KLocalizedString bps = ki18nc("Bytes per second unit symbol", "%1 B/s");
-    const static KLocalizedString Kbps = ki18nc("Kilobytes per second unit symbol", "%1 KiB/s");
-    const static KLocalizedString Mbps = ki18nc("Megabytes per second unit symbol", "%1 MiB/s");
-    const static KLocalizedString Gbps = ki18nc("Gigabytes per second unit symbol", "%1 GiB/s");
-    const static KLocalizedString Tbps = ki18nc("Terabytes per second unit symbol", "%1 TiB/s");
-    const static KLocalizedString Pbps = ki18nc("Petabytes per second unit symbol", "%1 PiB/s");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString B = ki18nc("Bytes unit symbol", "%1\u2009B");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString KiB = ki18nc("Kilobytes unit symbol", "%1\u202FKi\u200BB");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString MiB = ki18nc("Megabytes unit symbol", "%1\u202FMi\u200BB");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString GiB = ki18nc("Gigabytes unit symbol", "%1\u202FGi\u200BB");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString TiB = ki18nc("Terabytes unit symbol", "%1\u202FTi\u200BB");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString PiB = ki18nc("Petabytes unit symbol", "%1\u202FPi\u200BB");
 
-    const static KLocalizedString bitsps = ki18nc("Bits per second unit symbol", "%1 bps");
-    const static KLocalizedString Kbitsps = ki18nc("Kilobits per second unit symbol", "%1 Kbps");
-    const static KLocalizedString Mbitsps = ki18nc("Megabits per second unit symbol", "%1 Mbps");
-    const static KLocalizedString Gbitsps = ki18nc("Gigabits per second unit symbol", "%1 Gbps");
-    const static KLocalizedString Tbitsps = ki18nc("Terabits per second unit symbol", "%1 Tbps");
-    const static KLocalizedString Pbitsps = ki18nc("Petabits per second unit symbol", "%1 Pbps");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString bps = ki18nc("Bytes per second unit symbol", "%1\u2009B/s");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Kbps = ki18nc("Kilobytes per second unit symbol", "%1\u202FKi\u200BB/s");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Mbps = ki18nc("Megabytes per second unit symbol", "%1\u202FMi\u200BB/s");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Gbps = ki18nc("Gigabytes per second unit symbol", "%1\u202FGi\u200BB/s");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Tbps = ki18nc("Terabytes per second unit symbol", "%1\u202FTi\u200BB/s");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Pbps = ki18nc("Petabytes per second unit symbol", "%1\u202FPi\u200BB/s");
 
-    const static KLocalizedString Hz = ki18nc("Hertz unit symbol", "%1 Hz");
-    const static KLocalizedString kHz = ki18nc("Kilohertz unit symbol", "%1 kHz");
-    const static KLocalizedString MHz = ki18nc("Megahertz unit symbol", "%1 MHz");
-    const static KLocalizedString GHz = ki18nc("Gigahertz unit symbol", "%1 GHz");
-    const static KLocalizedString THz = ki18nc("Terahertz unit symbol", "%1 THz");
-    const static KLocalizedString PHz = ki18nc("Petahertz unit symbol", "%1 PHz");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString bitsps = ki18nc("Bits per second unit symbol", "%1\u2009bps");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Kbitsps = ki18nc("Kilobits per second unit symbol", "%1\u202Fk\u200Bbps");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Mbitsps = ki18nc("Megabits per second unit symbol", "%1\u202FM\u200Bbps");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Gbitsps = ki18nc("Gigabits per second unit symbol", "%1\u202FG\u200Bbps");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Tbitsps = ki18nc("Terabits per second unit symbol", "%1\u202FT\u200Bbps");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString Pbitsps = ki18nc("Petabits per second unit symbol", "%1\u202FP\u200Bbps");
 
-    const static KLocalizedString V = ki18nc("Volts unit symbol", "%1 V");
-    const static KLocalizedString kV = ki18nc("Kilovolts unit symbol", "%1 kV");
-    const static KLocalizedString MV = ki18nc("Megavolts unit symbol", "%1 MV");
-    const static KLocalizedString GV = ki18nc("Gigavolts unit symbol", "%1 GV");
-    const static KLocalizedString TV = ki18nc("Teravolts unit symbol", "%1 TV");
-    const static KLocalizedString PV = ki18nc("Petavolts unit symbol", "%1 PV");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString Hz = ki18nc("Hertz unit symbol", "%1\u2009Hz");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString kHz = ki18nc("Kilohertz unit symbol", "%1\u202Fk\u200BHz");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString MHz = ki18nc("Megahertz unit symbol", "%1\u202FM\u200BHz");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString GHz = ki18nc("Gigahertz unit symbol", "%1\u202FG\u200BHz");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString THz = ki18nc("Terahertz unit symbol", "%1\u202FT\u200BHz");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString PHz = ki18nc("Petahertz unit symbol", "%1\u202FP\u200BHz");
 
-    const static KLocalizedString W = ki18nc("Watts unit symbol", "%1 W");
-    const static KLocalizedString kW = ki18nc("Kilowatts unit symbol", "%1 kW");
-    const static KLocalizedString MW = ki18nc("Megawatts unit symbol", "%1 MW");
-    const static KLocalizedString GW = ki18nc("Gigawatts unit symbol", "%1 GW");
-    const static KLocalizedString TW = ki18nc("Terawatts unit symbol", "%1 TW");
-    const static KLocalizedString PW = ki18nc("Petawatts unit symbol", "%1 PW");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString V = ki18nc("Volts unit symbol", "%1\u2009V");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString kV = ki18nc("Kilovolts unit symbol", "%1\u202Fk\u200BV");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString MV = ki18nc("Megavolts unit symbol", "%1\u202FM\u200BV");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString GV = ki18nc("Gigavolts unit symbol", "%1\u202FG\u200BV");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString TV = ki18nc("Teravolts unit symbol", "%1\u202FT\u200BV");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString PV = ki18nc("Petavolts unit symbol", "%1\u202FP\u200BV");
 
-    const static KLocalizedString Wh = ki18nc("Watt-hours unit symbol", "%1 Wh");
-    const static KLocalizedString kWh = ki18nc("Kilowatt-hours unit symbol", "%1 kWh");
-    const static KLocalizedString MWh = ki18nc("Megawatt-hours unit symbol", "%1 MWh");
-    const static KLocalizedString GWh = ki18nc("Gigawatt-hours unit symbol", "%1 GWh");
-    const static KLocalizedString TWh = ki18nc("Terawatt-hours unit symbol", "%1 TWh");
-    const static KLocalizedString PWh = ki18nc("Petawatt-hours unit symbol", "%1 PWh");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString W = ki18nc("Watts unit symbol", "%1\u2009W");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString kW = ki18nc("Kilowatts unit symbol", "%1\u202Fk\u200BW");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString MW = ki18nc("Megawatts unit symbol", "%1\u202FM\u200BW");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString GW = ki18nc("Gigawatts unit symbol", "%1\u202FG\u200BW");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString TW = ki18nc("Terawatts unit symbol", "%1\u202FT\u200BW");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString PW = ki18nc("Petawatts unit symbol", "%1\u202FP\u200BW");
 
-    const static KLocalizedString A = ki18nc("Ampere unit symbol", "%1 A");
-    const static KLocalizedString kA = ki18nc("Kiloamperes unit symbol", "%1 kA");
-    const static KLocalizedString MA = ki18nc("Megaamperes unit symbol", "%1 MA");
-    const static KLocalizedString GA = ki18nc("Gigaamperes unit symbol", "%1 GA");
-    const static KLocalizedString TA = ki18nc("Teraamperes unit symbol", "%1 TA");
-    const static KLocalizedString PA = ki18nc("Petaamperes unit symbol", "%1 PA");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString Wh = ki18nc("Watt-hours unit symbol", "%1\u2009Wh");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString kWh = ki18nc("Kilowatt-hours unit symbol", "%1\u202Fk\u200BWh");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString MWh = ki18nc("Megawatt-hours unit symbol", "%1\u202FM\u200BWh");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString GWh = ki18nc("Gigawatt-hours unit symbol", "%1\u202FG\u200BWh");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString TWh = ki18nc("Terawatt-hours unit symbol", "%1\u202FT\u200BWh");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString PWh = ki18nc("Petawatt-hours unit symbol", "%1\u202FP\u200BWh");
 
-    const static KLocalizedString percent = ki18nc("Percent unit", "%1%");
-    const static KLocalizedString RPM = ki18nc("Revolutions per minute unit symbol", "%1 RPM");
-    const static KLocalizedString C = ki18nc("Celsius unit symbol", "%1°C");
-    const static KLocalizedString dBm = ki18nc("Decibels unit symbol", "%1 dBm");
-    const static KLocalizedString s = ki18nc("Seconds unit symbol", "%1s");
-    const static KLocalizedString rate = ki18nc("Rate unit symbol", "%1 s⁻¹");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString A = ki18nc("Ampere unit symbol", "%1\u2009A");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString kA = ki18nc("Kiloamperes unit symbol", "%1\u202Fk\u200BA");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString MA = ki18nc("Megaamperes unit symbol", "%1\u202FM\u200BA");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString GA = ki18nc("Gigaamperes unit symbol", "%1\u202FG\u200BA");
+    // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString TA = ki18nc("Teraamperes unit symbol", "%1\u202FT\u200BA");
+    // // i18n: Use \u202F (thin non-break space) to separate value and prefixed unit; Use \u200B (zero-width space) to separate prefix and unit.
+    const static KLocalizedString PA = ki18nc("Petaamperes unit symbol", "%1\u202FP\u200BA");
+
+    // i18n: Use \u200B (zero width space) to separate value and unit
+    const static KLocalizedString percent = ki18nc("Percent unit", "%1\u200B%");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString RPM = ki18nc("Revolutions per minute unit symbol", "%1\u2009RPM");
+    // i18n: Use \u200B (zero width space) to separate value and unit
+    const static KLocalizedString C = ki18nc("Celsius unit symbol", "%1\u200B°C");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString dBm = ki18nc("Decibels unit symbol", "%1\u2009dBm");
+    // i18n: Use \u200B (zero width space) to separate value and unit
+    const static KLocalizedString s = ki18nc("Seconds unit symbol", "%1\u200Bs");
+    // i18n: Use \u2009 (thin space) to separate value and unit
+    const static KLocalizedString rate = ki18nc("Rate unit symbol", "%1\u2009s⁻¹");
     const static KLocalizedString unitless = ki18nc("Unitless", "%1");
 
     switch (unit) {
@@ -376,7 +439,7 @@ static Unit adjustedUnit(qreal value, Unit unit, MetricPrefix prefix)
         }
     }
 
-    const Unit newUnit = Unit(prefix + baseUnit);
+    const Unit newUnit = Unit(int(prefix) + int(baseUnit));
     // If there is no prefixed unit,
     // don't overflow into the following unrelated units.
     if (unitBase(newUnit) != baseUnit) {
@@ -386,7 +449,7 @@ static Unit adjustedUnit(qreal value, Unit unit, MetricPrefix prefix)
     return newUnit;
 }
 
-static QString formatNumber(const QVariant &value, Unit unit, MetricPrefix prefix, FormatOptions options)
+static QString formatNumber(const QVariant &value, Unit unit, MetricPrefix prefix, FormatOptions options, int precision)
 {
     qreal amount = value.toDouble();
 
@@ -399,7 +462,9 @@ static QString formatNumber(const QVariant &value, Unit unit, MetricPrefix prefi
         amount /= std::pow(unitOrder(unit), adjusted - unit);
     }
 
-    const int precision = (value.type() != QVariant::Double && adjusted <= unit) ? 0 : 1;
+    if (precision < 0) {
+        precision = (value.typeId() != QMetaType::Double && adjusted <= unit) ? 0 : 1;
+    }
     const QString text = QLocale().toString(amount, 'f', precision);
 
     return unitFormat(adjusted).subs(text).toString();
@@ -449,7 +514,7 @@ KLocalizedString Formatter::localizedString(const QVariant &value, Unit unit, Me
     return unitFormat(adjusted);
 }
 
-QString Formatter::formatValue(const QVariant &value, Unit unit, MetricPrefix targetPrefix, FormatOptions options)
+QString Formatter::formatValue(const QVariant &value, Unit unit, MetricPrefix targetPrefix, FormatOptions options, int precision)
 {
     switch (unit) {
     case UnitByte:
@@ -506,14 +571,14 @@ QString Formatter::formatValue(const QVariant &value, Unit unit, MetricPrefix ta
     case UnitRpm:
     case UnitCelsius:
     case UnitDecibelMilliWatts:
-        return formatNumber(value, unit, targetPrefix, options);
+        return formatNumber(value, unit, targetPrefix, options, precision);
 
     case UnitBootTimestamp:
         return formatBootTimestamp(value);
     case UnitTime:
         return formatTime(value);
     case UnitNone:
-        return formatNumber(value, unit, MetricPrefix::MetricPrefixUnity, options);
+        return formatNumber(value, unit, MetricPrefix::MetricPrefixUnity, options, precision);
     case UnitTicks:
         return formatTicks(value);
 
@@ -668,7 +733,7 @@ qreal Formatter::maximumLength(Unit unit, const QFont &font)
         maximum = formatValue(order - 0.5, UnitMegaHertz, MetricPrefixMega);
         break;
     case UnitPercent:
-        maximum = formatValue(9999.9, UnitPercent);
+        maximum = formatValue(999.9, UnitPercent);
         break;
     default:
         return -1.0;
